%   DESCRIPTION
%   This script designed for demonstration of solving optimization problem
%   about switching velocity direction of pendulum in viscous media

%   PARAMETERS
%   Driving torque limitation: Umax belongs to (0, infinity)
Umax = 1;
%   Initial condition: OmegaDot_0 belongs to [0, Umax)
OmegaDot_0 = .8;
%   Segment of interest
omega_2 = -Umax+0.01:0.01:Umax-0.01;

%   PHASE PLANE
%   Phase variables are displacement and velocity - omega_1 and omega_2 respectively
figure(1);
hold on;
%   Plot several phase curves
for offset=-2:0.5:2
    plot(omega_2, -omega_2-Umax*log(Umax-omega_2)+offset+Umax*log(Umax), 'Color', 'red');
    plot(omega_2, -omega_2+Umax*log(Umax+omega_2)+offset-Umax*log(Umax));
end
hold off
%   Format the plot
title('Phase plane', 'FontSize', 10, 'FontWeight', 'bold')
axis([-Umax, Umax, -2, 2])
xlabel('\omega_1', 'FontSize', 14, 'FontWeight', 'bold')
ylabel('\omega_2', 'FontSize', 14, 'FontWeight', 'bold')
set(gca, 'CameraUpVector', [1,0,0]);
set(gca, 'YDir', 'reverse');
set(gca, 'XGrid', 'on');
set(gca, 'YGrid', 'on');

%   VELOCITY SWITCH IN PHASE PLANE
%   Switch time
Tsw = -log((Umax-sqrt(Umax^2-(Umax-OmegaDot_0)^2*exp(2*OmegaDot_0/Umax)))/(Umax-OmegaDot_0));
disp(sprintf('Switch time\n\tTsw = %f', Tsw))
%   Time of switching process
T = 2*OmegaDot_0/Umax+2*Tsw;
disp(sprintf('Time of switching process\n\tT = %f', T))
%   Ordinate of switch in phase plane
omega_2sw = (Umax-OmegaDot_0)*exp(-Tsw)-Umax;
%   Segments of applying +/-Umax
segment0 = omega_2sw:0.01:-OmegaDot_0;
segment1 = omega_2sw:0.01:OmegaDot_0;
%   Defining of arbitrary constants
C01 = -OmegaDot_0-Umax*log(Umax-OmegaDot_0);
C02 = OmegaDot_0+Umax*log(Umax-OmegaDot_0);
%   Plot relative phase curves and highlight switching process
figure(2)
hold on;
plot(omega_2, -omega_2+Umax*log(Umax+omega_2)+C01);
plot(segment0, -segment0+Umax*log(Umax+segment0)+C01, 'Color', 'green', 'LineWidth', 2);
plot(omega_2, -omega_2-Umax*log(Umax-omega_2)+C02, 'Color', 'red');
plot(segment1, -segment1-Umax*log(Umax-segment1)+C02, 'Color', 'green', 'LineWidth', 2);
%   Switch point
plot(omega_2sw, -omega_2sw+Umax*log(Umax+omega_2sw)+C01, '.', 'Color', 'black');
hold off;
%   Format the plot
title('Velocity switch', 'FontSize', 10, 'FontWeight', 'bold');
xlabel('\omega_1', 'FontSize', 14, 'FontWeight', 'bold')
ylabel('\omega_2', 'FontSize', 14, 'FontWeight', 'bold')
set(gca, 'CameraUpVector', [1,0,0]);
set(gca, 'YDir', 'reverse');
set(gca, 'XGrid', 'on');
set(gca, 'YGrid', 'on');

%   DISPLACEMENT PLOT
%   Time segments
t0 = 0:0.01:Tsw;
t1 = Tsw:0.01:T;
%   Defining of arbitrary constants
C01 = -Umax+OmegaDot_0;
C11 = Umax-OmegaDot_0;
C02 = -Umax+2*Umax*exp(Tsw)+OmegaDot_0;
C12 = -Umax-2*Umax*Tsw-OmegaDot_0;
%   Plot displacement
figure(3)
hold on
plot(t0, C01*exp(-t0)+C11-Umax*t0)
plot(t1, C02*exp(-t1)+C12+Umax*t1)
%   Switch point
plot(Tsw, C01*exp(-Tsw)+C11-Umax*Tsw, '.', 'Color', 'black');
hold off
%   Format the plot
title('Displacement', 'FontSize', 10, 'FontWeight', 'bold')
xlabel('t', 'FontSize', 14, 'FontWeight', 'bold')
ylabel('\Omega', 'FontSize', 14, 'FontWeight', 'bold')
set(gca, 'XGrid', 'on');
set(gca, 'YGrid', 'on');

%   VELOCITY PLOT
%   Plot velocity
figure(4)
hold on
plot(t0, -C01*exp(-t0)-Umax)
plot(t1, -C02*exp(-t1)+Umax)
%   Switch point
plot(Tsw, -C01*exp(-Tsw)-Umax, '.', 'Color', 'black');
hold off
%   Format the plot
title('Velocity', 'FontSize', 10, 'FontWeight', 'bold')
xlabel('t', 'FontSize', 14, 'FontWeight', 'bold')
ylabel('\Omega\prime', 'FontSize', 14, 'FontWeight', 'bold')
set(gca, 'XGrid', 'on');
set(gca, 'YGrid', 'on');

%   ACCELERATION PLOT
%   Plot acceleration
figure(5)
hold on
plot(t0, C01*exp(-t0))
plot(t1, C02*exp(-t1))
%   Discontinuity
plot([Tsw, Tsw], [C01*exp(-Tsw), C02*exp(-Tsw)]);
hold off
%   Format the plot
title('Acceleration', 'FontSize', 10, 'FontWeight', 'bold')
xlabel('t', 'FontSize', 14, 'FontWeight', 'bold')
ylabel('\Omega\prime\prime', 'FontSize', 14, 'FontWeight', 'bold')
set(gca, 'XGrid', 'on');
set(gca, 'YGrid', 'on');
